package org.eclipse.jdt.internal.compiler.parser;

package .tmp;

import org.eclipse.jdt.internal.compiler.ASTVisitor;
import org.eclipse.jdt.internal.compiler.ast.AbstractMethodDeclaration;
import org.eclipse.jdt.internal.compiler.ast.ConstructorDeclaration;
import org.eclipse.jdt.internal.compiler.ast.Initializer;
import org.eclipse.jdt.internal.compiler.ast.MethodDeclaration;
import org.eclipse.jdt.internal.compiler.ast.TypeDeclaration;
import org.eclipse.jdt.internal.compiler.impl.ReferenceContext;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.lookup.ClassScope;
import org.eclipse.jdt.internal.compiler.lookup.MethodScope;
import org.eclipse.jdt.internal.compiler.parser.Parser;
import org.eclipse.jdt.internal.compiler.parser.Scanner;

class MethodVisitor extends ASTVisitor {
  public ASTVisitor typeVisitor;
  
  TypeDeclaration enclosingType;
  
  TypeDeclaration[] types = new TypeDeclaration[0];
  
  int typePtr = -1;
  
  public void endVisit(ConstructorDeclaration constructorDeclaration, ClassScope scope) {
    endVisitMethod((AbstractMethodDeclaration)constructorDeclaration, scope);
  }
  
  public void endVisit(Initializer initializer, MethodScope scope) {
    if (initializer.block == null)
      return; 
    TypeDeclaration[] foundTypes = null;
    int length = 0;
    if (this.typePtr > -1) {
      length = this.typePtr + 1;
      foundTypes = new TypeDeclaration[length];
      System.arraycopy(this.types, 0, foundTypes, 0, length);
    } 
    ReferenceContext oldContext = Parser.this.referenceContext;
    Parser.this.recoveryScanner.resetTo(initializer.bodyStart, initializer.bodyEnd);
    Scanner oldScanner = Parser.this.scanner;
    Parser.this.scanner = (Scanner)Parser.this.recoveryScanner;
    Parser.this.parseStatements(
        (ReferenceContext)this.enclosingType, 
        initializer.bodyStart, 
        initializer.bodyEnd, 
        foundTypes, 
        Parser.this.compilationUnit);
    Parser.this.scanner = oldScanner;
    Parser.this.referenceContext = oldContext;
    for (int i = 0; i < length; i++)
      foundTypes[i].traverse(this.typeVisitor, (BlockScope)scope); 
  }
  
  public void endVisit(MethodDeclaration methodDeclaration, ClassScope scope) {
    endVisitMethod((AbstractMethodDeclaration)methodDeclaration, scope);
  }
  
  private void endVisitMethod(AbstractMethodDeclaration methodDeclaration, ClassScope scope) {
    TypeDeclaration[] foundTypes = null;
    int length = 0;
    if (this.typePtr > -1) {
      length = this.typePtr + 1;
      foundTypes = new TypeDeclaration[length];
      System.arraycopy(this.types, 0, foundTypes, 0, length);
    } 
    ReferenceContext oldContext = Parser.this.referenceContext;
    Parser.this.recoveryScanner.resetTo(methodDeclaration.bodyStart, methodDeclaration.bodyEnd);
    Scanner oldScanner = Parser.this.scanner;
    Parser.this.scanner = (Scanner)Parser.this.recoveryScanner;
    Parser.this.parseStatements(
        (ReferenceContext)methodDeclaration, 
        methodDeclaration.bodyStart, 
        methodDeclaration.bodyEnd, 
        foundTypes, 
        Parser.this.compilationUnit);
    Parser.this.scanner = oldScanner;
    Parser.this.referenceContext = oldContext;
    for (int i = 0; i < length; i++)
      foundTypes[i].traverse(this.typeVisitor, scope); 
  }
  
  public boolean visit(ConstructorDeclaration constructorDeclaration, ClassScope scope) {
    this.typePtr = -1;
    return true;
  }
  
  public boolean visit(Initializer initializer, MethodScope scope) {
    this.typePtr = -1;
    if (initializer.block == null)
      return false; 
    return true;
  }
  
  public boolean visit(MethodDeclaration methodDeclaration, ClassScope scope) {
    this.typePtr = -1;
    return true;
  }
  
  private boolean visit(TypeDeclaration typeDeclaration) {
    if (this.types.length <= ++this.typePtr) {
      int length = this.typePtr;
      System.arraycopy(this.types, 0, this.types = new TypeDeclaration[length * 2 + 1], 0, length);
    } 
    this.types[this.typePtr] = typeDeclaration;
    return false;
  }
  
  public boolean visit(TypeDeclaration typeDeclaration, BlockScope scope) {
    return visit(typeDeclaration);
  }
  
  public boolean visit(TypeDeclaration typeDeclaration, ClassScope scope) {
    return visit(typeDeclaration);
  }
}
